@@ -20,15 +20,18 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "pixman.h"
 #include "pixman-private.h"
 
+#define Alpha(x) ((x) >> 24)
+
 static void
 init_source_image (source_image_t *image)
 {
@@ -49,7 +52,7 @@ init_gradient (gradient_t     *gradient,
 	return FALSE;
 
     memcpy (gradient->stops, stops, n_stops * sizeof (pixman_gradient_stop_t));
-    
+
     gradient->n_stops = n_stops;
 
     gradient->stop_range = 0xffff;
@@ -73,13 +76,13 @@ static pixman_image_t *
 allocate_image (void)
 {
     pixman_image_t *image = malloc (sizeof (pixman_image_t));
-    
+
     if (image)
     {
 	image_common_t *common = &image->common;
 
-	pixman_region_init (&common->full_region);
-	pixman_region_init (&common->clip_region);
+	pixman_region32_init (&common->full_region);
+	pixman_region32_init (&common->clip_region);
 	common->src_clip = &common->full_region;
 	common->has_client_clip = FALSE;
 	common->transform = NULL;
@@ -98,7 +101,7 @@ allocate_image (void)
 }
 
 /* Ref Counting */
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_ref (pixman_image_t *image)
 {
     image->common.ref_count++;
@@ -107,7 +110,7 @@ pixman_image_ref (pixman_image_t *image)
 }
 
 /* returns TRUE when the image is freed */
-pixman_bool_t
+PIXMAN_EXPORT pixman_bool_t
 pixman_image_unref (pixman_image_t *image)
 {
     image_common_t *common = (image_common_t *)image;
@@ -116,8 +119,8 @@ pixman_image_unref (pixman_image_t *image)
 
     if (common->ref_count == 0)
     {
-	pixman_region_fini (&common->clip_region);
-	pixman_region_fini (&common->full_region);
+	pixman_region32_fini (&common->clip_region);
+	pixman_region32_fini (&common->full_region);
 
 	if (common->transform)
 	    free (common->transform);
@@ -132,7 +135,7 @@ pixman_image_unref (pixman_image_t *image)
 	if (image->type == BITS && image->bits.indexed)
 	    free (image->bits.indexed);
 #endif
-	
+
 #if 0
 	memset (image, 0xaa, sizeof (pixman_image_t));
 #endif
@@ -142,10 +145,10 @@ pixman_image_unref (pixman_image_t *image)
 		free (image->gradient.stops);
 	}
 
-	
+
 	if (image->type == BITS && image->bits.free_me)
 	    free (image->bits.free_me);
-	
+
 	free (image);
 
 	return TRUE;
@@ -155,22 +158,22 @@ pixman_image_unref (pixman_image_t *image)
 }
 
 /* Constructors */
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_solid_fill (pixman_color_t *color)
 {
     pixman_image_t *img = allocate_image();
     if (!img)
 	return NULL;
-    
+
     init_source_image (&img->solid.common);
-    
+
     img->type = SOLID;
     img->solid.color = color_to_uint32 (color);
 
     return img;
 }
 
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_linear_gradient (pixman_point_fixed_t         *p1,
 				     pixman_point_fixed_t         *p2,
 				     const pixman_gradient_stop_t *stops,
@@ -180,14 +183,14 @@ pixman_image_create_linear_gradient (pixman_point_fixed_t         *p1,
     linear_gradient_t *linear;
 
     return_val_if_fail (n_stops >= 2, NULL);
-    
+
     image = allocate_image();
-    
+
     if (!image)
 	return NULL;
 
     linear = &image->linear;
-    
+
     if (!init_gradient (&linear->common, stops, n_stops))
     {
 	free (image);
@@ -203,7 +206,7 @@ pixman_image_create_linear_gradient (pixman_point_fixed_t         *p1,
 }
 
 
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_radial_gradient (pixman_point_fixed_t         *inner,
 				     pixman_point_fixed_t         *outer,
 				     pixman_fixed_t                inner_radius,
@@ -215,7 +218,7 @@ pixman_image_create_radial_gradient (pixman_point_fixed_t         *inner,
     radial_gradient_t *radial;
 
     return_val_if_fail (n_stops >= 2, NULL);
-    
+
     image = allocate_image();
 
     if (!image)
@@ -230,7 +233,7 @@ pixman_image_create_radial_gradient (pixman_point_fixed_t         *inner,
     }
 
     image->type = RADIAL;
-    
+
     radial->c1.x = inner->x;
     radial->c1.y = inner->y;
     radial->c1.radius = inner_radius;
@@ -243,11 +246,11 @@ pixman_image_create_radial_gradient (pixman_point_fixed_t         *inner,
     radial->A = (radial->cdx * radial->cdx
 		 + radial->cdy * radial->cdy
 		 - radial->dr  * radial->dr);
-    
+
     return image;
 }
 
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_conical_gradient (pixman_point_fixed_t *center,
 				      pixman_fixed_t angle,
 				      const pixman_gradient_stop_t *stops,
@@ -260,7 +263,7 @@ pixman_image_create_conical_gradient (pixman_point_fixed_t *center,
 	return NULL;
 
     conical = &image->conical;
-    
+
     if (!init_gradient (&conical->common, stops, n_stops))
     {
 	free (image);
@@ -283,9 +286,32 @@ create_bits (pixman_format_code_t format,
     int stride;
     int buf_size;
     int bpp;
-    
+
+    /* what follows is a long-winded way, avoiding any possibility of integer
+     * overflows, of saying:
+     * stride = ((width * bpp + FB_MASK) >> FB_SHIFT) * sizeof (uint32_t);
+     */
+
     bpp = PIXMAN_FORMAT_BPP (format);
-    stride = ((width * bpp + FB_MASK) >> FB_SHIFT) * sizeof (uint32_t);
+    if (pixman_multiply_overflows_int (width, bpp))
+	return NULL;
+
+    stride = width * bpp;
+    if (pixman_addition_overflows_int (stride, FB_MASK))
+	return NULL;
+
+    stride += FB_MASK;
+    stride >>= FB_SHIFT;
+
+#if FB_SHIFT < 2
+    if (pixman_multiply_overflows_int (stride, sizeof (uint32_t)))
+	return NULL;
+#endif
+    stride *= sizeof (uint32_t);
+
+    if (pixman_multiply_overflows_int (height, stride))
+	return NULL;
+
     buf_size = height * stride;
 
     if (rowstride_bytes)
@@ -297,20 +323,20 @@ create_bits (pixman_format_code_t format,
 static void
 reset_clip_region (pixman_image_t *image)
 {
-    pixman_region_fini (&image->common.clip_region);
-    
+    pixman_region32_fini (&image->common.clip_region);
+
     if (image->type == BITS)
     {
-	pixman_region_init_rect (&image->common.clip_region, 0, 0,
-				 image->bits.width, image->bits.height);	
+	pixman_region32_init_rect (&image->common.clip_region, 0, 0,
+				   image->bits.width, image->bits.height);
     }
     else
     {
-	pixman_region_init (&image->common.clip_region);
+	pixman_region32_init (&image->common.clip_region);
     }
 }
 
-pixman_image_t *
+PIXMAN_EXPORT pixman_image_t *
 pixman_image_create_bits (pixman_format_code_t  format,
 			  int                   width,
 			  int                   height,
@@ -320,22 +346,25 @@ pixman_image_create_bits (pixman_format_code_t  format,
     pixman_image_t *image;
     uint32_t *free_me = NULL;
 
-    /* must be a whole number of uint32_t's 
+    /* must be a whole number of uint32_t's
      */
     return_val_if_fail (bits == NULL ||
-			(rowstride_bytes % sizeof (uint32_t)) == 0, NULL); 
+			(rowstride_bytes % sizeof (uint32_t)) == 0, NULL);
 
-    if (!bits)
+    if (!bits && width && height)
     {
 	free_me = bits = create_bits (format, width, height, &rowstride_bytes);
 	if (!bits)
 	    return NULL;
     }
-    
+
     image = allocate_image();
 
-    if (!image)
+    if (!image) {
+	if (free_me)
+	    free (free_me);
 	return NULL;
+    }
     
     image->type = BITS;
     image->bits.format = format;
@@ -343,21 +372,40 @@ pixman_image_create_bits (pixman_format_code_t  format,
     image->bits.height = height;
     image->bits.bits = bits;
     image->bits.free_me = free_me;
-    
-    image->bits.rowstride = rowstride_bytes / sizeof (uint32_t); /* we store it in number
+
+    image->bits.rowstride = rowstride_bytes / (int) sizeof (uint32_t); /* we store it in number
 								  * of uint32_t's
 								  */
     image->bits.indexed = NULL;
 
-    pixman_region_fini (&image->common.full_region);
-    pixman_region_init_rect (&image->common.full_region, 0, 0,
-			     image->bits.width, image->bits.height);
+    pixman_region32_fini (&image->common.full_region);
+    pixman_region32_init_rect (&image->common.full_region, 0, 0,
+			       image->bits.width, image->bits.height);
 
     reset_clip_region (image);
     return image;
 }
 
-pixman_bool_t
+PIXMAN_EXPORT pixman_bool_t
+pixman_image_set_clip_region32 (pixman_image_t *image,
+				pixman_region32_t *region)
+{
+    image_common_t *common = (image_common_t *)image;
+
+    if (region)
+    {
+	return pixman_region32_copy (&common->clip_region, region);
+    }
+    else
+    {
+	reset_clip_region (image);
+
+	return TRUE;
+    }
+}
+
+
+PIXMAN_EXPORT pixman_bool_t
 pixman_image_set_clip_region (pixman_image_t    *image,
 			      pixman_region16_t *region)
 {
@@ -365,26 +413,26 @@ pixman_image_set_clip_region (pixman_image_t    *image,
 
     if (region)
     {
-	return pixman_region_copy (&common->clip_region, region);
+	return pixman_region32_copy_from_region16 (&common->clip_region, region);
     }
     else
     {
 	reset_clip_region (image);
-	
+
 	return TRUE;
     }
 }
 
 /* Sets whether the clip region includes a clip region set by the client
  */
-void
+PIXMAN_EXPORT void
 pixman_image_set_has_client_clip (pixman_image_t *image,
 				  pixman_bool_t	  client_clip)
 {
     image->common.has_client_clip = client_clip;
 }
 
-pixman_bool_t
+PIXMAN_EXPORT pixman_bool_t
 pixman_image_set_transform (pixman_image_t           *image,
 			    const pixman_transform_t *transform)
 {
@@ -395,7 +443,7 @@ pixman_image_set_transform (pixman_image_t           *image,
 	  { 0, 0, pixman_fixed_1 }
 	}
     };
-    
+
     image_common_t *common = (image_common_t *)image;
 
     if (common->transform == transform)
@@ -407,7 +455,7 @@ pixman_image_set_transform (pixman_image_t           *image,
 	common->transform = NULL;
 	return TRUE;
     }
-    
+
     if (common->transform == NULL)
 	common->transform = malloc (sizeof (pixman_transform_t));
     if (common->transform == NULL)
@@ -418,14 +466,14 @@ pixman_image_set_transform (pixman_image_t           *image,
     return TRUE;
 }
 
-void
+PIXMAN_EXPORT void
 pixman_image_set_repeat (pixman_image_t  *image,
 			 pixman_repeat_t  repeat)
 {
     image->common.repeat = repeat;
 }
 
-pixman_bool_t 
+PIXMAN_EXPORT pixman_bool_t
 pixman_image_set_filter (pixman_image_t       *image,
 			 pixman_filter_t       filter,
 			 const pixman_fixed_t *params,
@@ -449,7 +497,7 @@ pixman_image_set_filter (pixman_image_t       *image,
     }
 
     common->filter = filter;
-	
+
     if (common->filter_params)
 	free (common->filter_params);
 
@@ -458,7 +506,7 @@ pixman_image_set_filter (pixman_image_t       *image,
     return TRUE;
 }
 
-void
+PIXMAN_EXPORT void
 pixman_image_set_source_clipping (pixman_image_t  *image,
 				  pixman_bool_t    source_clipping)
 {
@@ -474,7 +522,7 @@ pixman_image_set_source_clipping (pixman_image_t  *image,
  * copy the content of indexed. Doing this copying is simply
  * way, way too expensive.
  */
-void
+PIXMAN_EXPORT void
 pixman_image_set_indexed (pixman_image_t	 *image,
 			  const pixman_indexed_t *indexed)
 {
@@ -483,14 +531,14 @@ pixman_image_set_indexed (pixman_image_t	 *image,
     bits->indexed = indexed;
 }
 
-void
+PIXMAN_EXPORT void
 pixman_image_set_alpha_map (pixman_image_t *image,
 			    pixman_image_t *alpha_map,
 			    int16_t         x,
 			    int16_t         y)
 {
     image_common_t *common = (image_common_t *)image;
-    
+
     return_if_fail (!alpha_map || alpha_map->type == BITS);
 
     if (common->alpha_map != (bits_image_t *)alpha_map)
@@ -508,7 +556,7 @@ pixman_image_set_alpha_map (pixman_image_t *image,
     common->alpha_origin.y = y;
 }
 
-void
+PIXMAN_EXPORT void
 pixman_image_set_component_alpha   (pixman_image_t       *image,
 				    pixman_bool_t         component_alpha)
 {
@@ -516,7 +564,7 @@ pixman_image_set_component_alpha   (pixman_image_t       *image,
 }
 
 
-void
+PIXMAN_EXPORT void
 pixman_image_set_accessors (pixman_image_t             *image,
 			    pixman_read_memory_func_t	read_func,
 			    pixman_write_memory_func_t	write_func)
@@ -527,7 +575,7 @@ pixman_image_set_accessors (pixman_image_t             *image,
     image->common.write_func = write_func;
 }
 
-uint32_t *
+PIXMAN_EXPORT uint32_t *
 pixman_image_get_data (pixman_image_t *image)
 {
     if (image->type == BITS)
@@ -536,7 +584,7 @@ pixman_image_get_data (pixman_image_t *image)
     return NULL;
 }
 
-int
+PIXMAN_EXPORT int
 pixman_image_get_width (pixman_image_t *image)
 {
     if (image->type == BITS)
@@ -545,7 +593,7 @@ pixman_image_get_width (pixman_image_t *image)
     return 0;
 }
 
-int
+PIXMAN_EXPORT int
 pixman_image_get_height (pixman_image_t *image)
 {
     if (image->type == BITS)
@@ -554,16 +602,16 @@ pixman_image_get_height (pixman_image_t *image)
     return 0;
 }
 
-int
+PIXMAN_EXPORT int
 pixman_image_get_stride (pixman_image_t *image)
 {
     if (image->type == BITS)
-	return image->bits.rowstride * sizeof (uint32_t);
+	return image->bits.rowstride * (int) sizeof (uint32_t);
 
     return 0;
 }
 
-int
+PIXMAN_EXPORT int
 pixman_image_get_depth (pixman_image_t *image)
 {
     if (image->type == BITS)
@@ -572,7 +620,7 @@ pixman_image_get_depth (pixman_image_t *image)
     return 0;
 }
 
-pixman_bool_t
+static pixman_bool_t
 color_to_pixel (pixman_color_t *color,
 		uint32_t       *pixel,
 		pixman_format_code_t format)
@@ -583,13 +631,15 @@ color_to_pixel (pixman_color_t *color,
 	  format == PIXMAN_x8r8g8b8	||
 	  format == PIXMAN_a8b8g8r8	||
 	  format == PIXMAN_x8b8g8r8	||
+	  format == PIXMAN_b8g8r8a8	||
+	  format == PIXMAN_b8g8r8x8	||
 	  format == PIXMAN_r5g6b5	||
 	  format == PIXMAN_b5g6r5	||
 	  format == PIXMAN_a8))
     {
 	return FALSE;
     }
-    
+
     if (PIXMAN_FORMAT_TYPE (format) == PIXMAN_TYPE_ABGR)
     {
 	c = ((c & 0xff000000) >>  0) |
@@ -597,6 +647,13 @@ color_to_pixel (pixman_color_t *color,
 	    ((c & 0x0000ff00) >>  0) |
 	    ((c & 0x000000ff) << 16);
     }
+    if (PIXMAN_FORMAT_TYPE (format) == PIXMAN_TYPE_BGRA)
+    {
+	c = ((c & 0xff000000) >> 24) |
+	    ((c & 0x00ff0000) >>  8) |
+	    ((c & 0x0000ff00) <<  8) |
+	    ((c & 0x000000ff) << 24);
+    }
 
     if (format == PIXMAN_a8)
 	c = c >> 24;
@@ -608,12 +665,12 @@ color_to_pixel (pixman_color_t *color,
     printf ("color: %x %x %x %x\n", color->alpha, color->red, color->green, color->blue);
     printf ("pixel: %x\n", c);
 #endif
-    
+
     *pixel = c;
     return TRUE;
 }
 
-pixman_bool_t
+PIXMAN_EXPORT pixman_bool_t
 pixman_image_fill_rectangles (pixman_op_t		    op,
 			      pixman_image_t		   *dest,
 			      pixman_color_t		   *color,
@@ -623,7 +680,7 @@ pixman_image_fill_rectangles (pixman_op_t		    op,
     pixman_image_t *solid;
     pixman_color_t c;
     int i;
-    
+
     if (color->alpha == 0xffff)
     {
 	if (op == PIXMAN_OP_OVER)
@@ -638,40 +695,44 @@ pixman_image_fill_rectangles (pixman_op_t		    op,
 	c.alpha = 0;
 
 	color = &c;
-	
+
 	op = PIXMAN_OP_SRC;
     }
 
     if (op == PIXMAN_OP_SRC)
     {
 	uint32_t pixel;
-	
+
 	if (color_to_pixel (color, &pixel, dest->bits.format))
 	{
 	    for (i = 0; i < n_rects; ++i)
 	    {
-		pixman_region16_t fill_region;
+		pixman_region32_t fill_region;
 		int n_boxes, j;
-		pixman_box16_t *boxes;
-		
-		pixman_region_init_rect (&fill_region, rects[i].x, rects[i].y, rects[i].width, rects[i].height);
-		pixman_region_intersect (&fill_region, &fill_region, &dest->common.clip_region);
+		pixman_box32_t *boxes;
 
-		boxes = pixman_region_rectangles (&fill_region, &n_boxes);
+		pixman_region32_init_rect (&fill_region, rects[i].x, rects[i].y, rects[i].width, rects[i].height);
+		if (!pixman_region32_intersect (&fill_region,
+						&fill_region,
+						&dest->common.clip_region))
+		    return FALSE;
+
+
+		boxes = pixman_region32_rectangles (&fill_region, &n_boxes);
 		for (j = 0; j < n_boxes; ++j)
 		{
-		    const pixman_box16_t *box = &(boxes[j]);
+		    const pixman_box32_t *box = &(boxes[j]);
 		    pixman_fill (dest->bits.bits, dest->bits.rowstride, PIXMAN_FORMAT_BPP (dest->bits.format),
 				 box->x1, box->y1, box->x2 - box->x1, box->y2 - box->y1,
 				 pixel);
 		}
 
-		pixman_region_fini (&fill_region);
+		pixman_region32_fini (&fill_region);
 	    }
 	    return TRUE;
 	}
     }
-    
+
     solid = pixman_image_create_solid_fill (color);
     if (!solid)
 	return FALSE;
@@ -679,14 +740,104 @@ pixman_image_fill_rectangles (pixman_op_t		    op,
     for (i = 0; i < n_rects; ++i)
     {
 	const pixman_rectangle16_t *rect = &(rects[i]);
-	
+
 	pixman_image_composite (op, solid, NULL, dest,
 				0, 0, 0, 0,
 				rect->x, rect->y,
 				rect->width, rect->height);
     }
-    
+
     pixman_image_unref (solid);
 
     return TRUE;
 }
+
+pixman_bool_t
+pixman_image_can_get_solid (pixman_image_t *image)
+{
+    if (image->type == SOLID)
+	return TRUE;
+
+    if (image->type != BITS	||
+	image->bits.width != 1	||
+	image->bits.height != 1)
+    {
+	return FALSE;
+    }
+
+    if (image->common.repeat != PIXMAN_REPEAT_NORMAL)
+	return FALSE;
+
+    switch (image->bits.format)
+    {
+    case PIXMAN_a8r8g8b8:
+    case PIXMAN_x8r8g8b8:
+    case PIXMAN_a8b8g8r8:
+    case PIXMAN_x8b8g8r8:
+    case PIXMAN_b8g8r8a8:
+    case PIXMAN_b8g8r8x8:
+    case PIXMAN_r8g8b8:
+    case PIXMAN_b8g8r8:
+    case PIXMAN_r5g6b5:
+    case PIXMAN_b5g6r5:
+	return TRUE;
+    default:
+	return FALSE;
+    }
+}
+
+pixman_bool_t
+pixman_image_is_opaque(pixman_image_t *image)
+{
+    int i = 0;
+    int gradientNumberOfColors = 0;
+
+    if(image->common.alpha_map)
+        return FALSE;
+
+    switch(image->type)
+    {
+    case BITS:
+        if(PIXMAN_FORMAT_A(image->bits.format))
+            return FALSE;
+        break;
+
+    case LINEAR:
+    case CONICAL:
+    case RADIAL:
+        gradientNumberOfColors = image->gradient.n_stops;
+        i=0;
+        while(i<gradientNumberOfColors)
+        {
+            if(image->gradient.stops[i].color.alpha != 0xffff)
+                return FALSE;
+            i++;
+        }
+        break;
+
+    case SOLID:
+         if(Alpha(image->solid.color) != 0xff)
+            return FALSE;
+        break;
+    }
+
+    /* Convolution filters can introduce translucency if the sum of the weights
+       is lower than 1. */
+    if (image->common.filter == PIXMAN_FILTER_CONVOLUTION)
+         return FALSE;
+
+    if (image->common.repeat == PIXMAN_REPEAT_NONE)
+    {
+        if (image->common.filter != PIXMAN_FILTER_NEAREST)
+            return FALSE;
+
+        if (image->common.transform)
+            return FALSE;
+
+	/* Gradients do not necessarily cover the entire compositing area */
+	if (image->type == LINEAR || image->type == CONICAL || image->type == RADIAL)
+	    return FALSE;
+    }
+
+     return TRUE;
+}
