@@ -67,7 +67,16 @@
 #endif
 
 #ifdef _MSC_VER
-#define inline __inline
+/* 'inline' is available only in C++ in MSVC */
+#   define inline __inline
+#   define force_inline __forceinline
+#elif defined __GNUC__ || (defined(__SUNPRO_C) && (__SUNPRO_C >= 0x590))
+#   define inline __inline__
+#   define force_inline __inline__ __attribute__ ((__always_inline__))
+#else
+# ifndef force_inline
+#  define force_inline inline
+# endif
 #endif
 
 #define FB_SHIFT    5
@@ -729,6 +738,9 @@ pixman_compute_composite_region32 (pixman_region32_t *	pRegion,
 /* GCC visibility */
 #if defined(__GNUC__) && __GNUC__ >= 4
 #define PIXMAN_EXPORT __attribute__ ((visibility("default")))
+/* Sun Studio 8 visibility */
+#elif defined(__SUNPRO_C) && (__SUNPRO_C >= 0x550)
+#define PIXMAN_EXPORT __global
 #else
 #define PIXMAN_EXPORT
 #endif
@@ -738,6 +750,9 @@ pixman_bool_t pixman_region32_copy_from_region16 (pixman_region32_t *dst,
 						  pixman_region16_t *src);
 pixman_bool_t pixman_region16_copy_from_region32 (pixman_region16_t *dst,
 						  pixman_region32_t *src);
+void pixman_region_internal_set_static_pointers (pixman_box16_t *empty_box,
+						 pixman_region16_data_t *empty_data,
+						 pixman_region16_data_t *broken_data);
 
 #ifdef PIXMAN_TIMING
 
